﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using com.epson.bank.driver;

namespace TMS_Sample
{
	public partial class TMS_SampleForm : Form
	{
		private CApp            m_objDriverControl  = null;
		private StructByStep    m_objConfigData     = null;

		private delegate DialogResult ShowMessageType(string text, string caption, MessageBoxButtons buttons, MessageBoxIcon icon);
		private delegate void ShowErrorMessageType(ErrorCode errResult);

		// Deligates to invoke
		ShowMessageType      ShowMessage      = null;
		ShowErrorMessageType ShowErrorMessage = null;

		// Step 1
		private bool m_bScanCancelError = false;

		public TMS_SampleForm()
		{
			InitializeComponent();
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (components != null)
				{
					components.Dispose();
				}
				// end process
				ErrorCode errRet = ErrorCode.SUCCESS;

				try
				{
					if (m_objDriverControl != null)
					{
						errRet = TerminateDriver();
						if (errRet != ErrorCode.SUCCESS)
						{
							ShowErrorMessage(errRet);
						}
					}
				}
				catch
				{
					base.Dispose(disposing);
				}
			}
			base.Dispose(disposing);
		}

		// Windows 64-bit has problem with FormLoad and unhandled exception, be carefull.
		// http://social.msdn.microsoft.com/Forums/vstudio/en-US/69a0b831-7782-4bd9-b910-25c85f18bceb/visual-studio-doesnt-break-on-unhandled-exception-with-windows-64bit?forum=vsdebug
		private void TMS_SampleForm_Load(object sender, EventArgs e)
		{
			// Create driver and data instance
			try
			{
				// Setting up deligates to invoke
				ShowMessage      = new ShowMessageType(ShowMessageImpl);
				ShowErrorMessage = new ShowErrorMessageType(ShowErrorMessageImpl);

				// Create driver and data instance
				m_objDriverControl = new CApp();
				m_objConfigData    = new StructByStep();

				if (InitializeDriver() != true)
				{
					this.Close();
					return;
				}

				// Initialize controls
				LoadResourceString();
			}
			catch (Exception ex)
			{
				ShowMessage(ex.Message, ConstComStr.CAPTION_06_000, MessageBoxButtons.OK, MessageBoxIcon.Error);
				this.Close();
				return;
			}
		}

		/////////////////////////////////////////////////////////////////
		// Procedure of DriveControl Callback
		/////////////////////////////////////////////////////////////////

		////////////////
		//// Step 1 ////
		////////////////

		private void CallbackProcProcessError(ErrorCode errResult)
		{
			Debug.Assert(m_objDriverControl != null);
			if (m_objDriverControl == null)
			{
				return;
			}

			ASB eStatus = new ASB();
			// Display message box correspondent to error
			switch( errResult )
			{
				case ErrorCode.ERR_COVER_OPEN:
					do
					{
						ShowMessage(ConstComStr.MSG_01_001, ConstComStr.CAPTION_01_001, MessageBoxButtons.OK, MessageBoxIcon.Error);
						m_objDriverControl.GetDeviceStatus(ref eStatus);
					} while ((eStatus & ASB.ASB_COVER_OPEN) == ASB.ASB_COVER_OPEN);
					m_objDriverControl.CancelError();
					break;

				case ErrorCode.ERR_PAPER_JAM:
					if (m_objConfigData.nRadio_ScanningMedia == ConstComVal.VAL_CONFIGDLG_RADIO_SM_CHECKPAPER)
					{
						do
						{
							ShowMessage(ConstComStr.MSG_02_000, ConstComStr.CAPTION_02_000, MessageBoxButtons.OK, MessageBoxIcon.Error);
							m_objDriverControl.GetDeviceStatus(ref eStatus);
						} while (((eStatus & ASB.ASB_EJECT_SENSOR_NO_PAPER) != ASB.ASB_EJECT_SENSOR_NO_PAPER)
										|| ((eStatus & ASB.ASB_SLIP_PAPER_SIZE) != ASB.ASB_SLIP_PAPER_SIZE)
										|| ((eStatus & ASB.ASB_PAPER_INTERMEDIATE) != ASB.ASB_PAPER_INTERMEDIATE));
						m_objDriverControl.CancelError();
					}
					else
					{
						ShowMessage(ConstComStr.MSG_02_000, ConstComStr.CAPTION_02_000, MessageBoxButtons.OK, MessageBoxIcon.Error);
						m_objDriverControl.CancelError();
					}
					break;

				case ErrorCode.ERR_ACCESS:
					ShowErrorMessage(errResult);
					m_objDriverControl.CancelError();
					break;

				case ErrorCode.ERR_MICR_BADDATA:
				case ErrorCode.ERR_MICR_NODATA:
				case ErrorCode.ERR_PRINT_DATA_UNRECEIVE:
					break;

				default:
					ShowErrorMessage(errResult);
					m_objDriverControl.CancelError();
					break;
			}
		}

		/////////////////////////////////////////////////////////////////
		// CTMS_SampleDlg message handlers
		/////////////////////////////////////////////////////////////////

		/////////////////////////////////////////////////////////////////
		// Button Event
		/////////////////////////////////////////////////////////////////

		////////////////
		//// Step 1 ////
		////////////////

		private void btnScan_Click(object sender, EventArgs e)
		{
			Debug.Assert(m_objDriverControl != null);
			if (m_objDriverControl == null)
			{
				return;
			}

			////////////////////////////////////////
			// Start scanning image

			{
				// Step 1
				ErrorCode errRet = m_objDriverControl.ScanCheck();
				if (errRet != ErrorCode.SUCCESS)
				{
					ShowErrorMessage(errRet);
					m_objDriverControl.CancelError();
					return;
				}
			}
		}

		private void btnExit_Click(object sender, EventArgs e)
		{
			Debug.Assert(m_objDriverControl != null);
			if (m_objDriverControl == null)
			{
				Close();
				return;
			}

			ErrorCode errRet = m_objDriverControl.ExitDevice();
			if (errRet != ErrorCode.SUCCESS)
			{
				ShowErrorMessage(errRet);
			}
			Close();
		}

		private void btnScanCancel_Click(object sender, EventArgs e)
		{
			Debug.Assert(m_objDriverControl != null);
			if (m_objDriverControl == null)
			{
				return;
			}

			ErrorCode errRet = m_objDriverControl.CancelScan();
			if (errRet != ErrorCode.SUCCESS)
			{
				m_bScanCancelError = true;
				ShowErrorMessage(errRet);
				m_objDriverControl.CancelError();
			}
		}

		/////////////////////////////////////////////////////////////////
		// Member functions
		/////////////////////////////////////////////////////////////////

		private bool InitializeDriver()
		{
			Debug.Assert(m_objDriverControl != null);
			if (m_objDriverControl == null)
			{
				return false;
			}

			ErrorCode errResult = ErrorCode.SUCCESS;
			DialogResult drRet = 0;
			string strResult = null;

			// Open device and register callback functions
			errResult = m_objDriverControl.InitDevice();
			while (errResult != ErrorCode.SUCCESS)
			{
				if (errResult == ErrorCode.ERR_UNKNOWN)
				{
					drRet = ShowMessage(ConstComStr.MSG_06_000, ConstComStr.CAPTION_06_000, MessageBoxButtons.OKCancel, MessageBoxIcon.Error);
				}
				else
				{
					strResult = GetErrorString(errResult);
					drRet = ShowMessage(ConstComStr.MSG_00_000, strResult, MessageBoxButtons.OKCancel, MessageBoxIcon.Error);
				}

				if (drRet == DialogResult.OK)
				{
					errResult = m_objDriverControl.InitDevice();
				}
				else
				{
					return false;
				}
			}

			// Specify callback function from DriverControl
			// Step 1
			m_objDriverControl.SetProcessErrorCallback(new CApp.CallbackProcProcessError(CallbackProcProcessError));

			return true;
		}

		private ErrorCode TerminateDriver()
		{
			Debug.Assert(m_objDriverControl != null);
			if (m_objDriverControl == null)
			{
				return ErrorCode.ERR_NO_MEMORY;
			}

			ErrorCode errRet = m_objDriverControl.ExitDevice();
			if (errRet != ErrorCode.SUCCESS)
			{
				return errRet;
			}

			return errRet;
		}

		private DialogResult ShowMessageImpl(string text, string caption, MessageBoxButtons buttons, MessageBoxIcon icon)
		{
			if (this.InvokeRequired)
			{
				return (DialogResult)this.Invoke(ShowMessage, new object[] { text, caption, buttons, icon });
			}
			return MessageBox.Show(text, caption, buttons, icon);
		}

		private void ShowErrorMessageImpl(ErrorCode errResult)
		{
			if (this.InvokeRequired)
			{
				this.Invoke(ShowErrorMessage, new object[] { errResult });
				return;
			}

			switch (errResult)
			{
				case ErrorCode.ERR_TYPE:
					MessageBox.Show(ConstComStr.MSG_03_000, ConstComStr.CAPTION_03_000, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_OPENED:
					MessageBox.Show(ConstComStr.MSG_03_001, ConstComStr.CAPTION_03_001, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_NO_PRINTER:
					MessageBox.Show(ConstComStr.MSG_03_002, ConstComStr.CAPTION_03_002, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_NO_TARGET:
					MessageBox.Show(ConstComStr.MSG_03_003, ConstComStr.CAPTION_03_003, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_NO_MEMORY:
					MessageBox.Show(ConstComStr.MSG_03_004, ConstComStr.CAPTION_03_004, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_HANDLE:
					MessageBox.Show(ConstComStr.MSG_03_005, ConstComStr.CAPTION_03_005, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_TIMEOUT:
					MessageBox.Show(ConstComStr.MSG_03_006, ConstComStr.CAPTION_03_006, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_ACCESS:
					MessageBox.Show(ConstComStr.MSG_03_007, ConstComStr.CAPTION_03_007, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_PARAM:
					MessageBox.Show(ConstComStr.MSG_03_008, ConstComStr.CAPTION_03_008, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_NOT_SUPPORT:
					MessageBox.Show(ConstComStr.MSG_03_009, ConstComStr.CAPTION_03_009, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_OFFLINE:
					MessageBox.Show(ConstComStr.MSG_03_010, ConstComStr.CAPTION_03_010, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_NOT_EPSON:
					MessageBox.Show(ConstComStr.MSG_03_011, ConstComStr.CAPTION_03_011, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_WITHOUT_CB:
					if (m_bScanCancelError == true)
					{
						MessageBox.Show(ConstComStr.MSG_03_012_01, ConstComStr.CAPTION_03_012, MessageBoxButtons.OK, MessageBoxIcon.Warning);
						m_bScanCancelError = false;
					}
					else
					{
						MessageBox.Show(ConstComStr.MSG_03_012_00, ConstComStr.CAPTION_03_012, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					}
					break;
				case ErrorCode.ERR_BUFFER_OVER_FLOW:
					MessageBox.Show(ConstComStr.MSG_03_013, ConstComStr.CAPTION_03_013, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_REGISTRY:
					MessageBox.Show(ConstComStr.MSG_03_014, ConstComStr.CAPTION_03_014, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_PAPERINSERT_TIMEOUT:
					MessageBox.Show(ConstComStr.MSG_03_015, ConstComStr.CAPTION_03_015, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_EXEC_FUNCTION:
					MessageBox.Show(ConstComStr.MSG_03_016, ConstComStr.CAPTION_03_016, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_EXEC_MICR:
					MessageBox.Show(ConstComStr.MSG_03_017, ConstComStr.CAPTION_03_017, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_EXEC_SCAN:
					MessageBox.Show(ConstComStr.MSG_03_018, ConstComStr.CAPTION_03_018, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_RESET:
					MessageBox.Show(ConstComStr.MSG_03_019, ConstComStr.CAPTION_03_019, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_ABORT:
					MessageBox.Show(ConstComStr.MSG_03_020, ConstComStr.CAPTION_03_020, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_MICR:
					MessageBox.Show(ConstComStr.MSG_03_021, ConstComStr.CAPTION_03_021, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_SCAN:
					MessageBox.Show(ConstComStr.MSG_03_022, ConstComStr.CAPTION_03_022, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_LINE_OVERFLOW:
					MessageBox.Show(ConstComStr.MSG_03_023, ConstComStr.CAPTION_03_023, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_PAPER_PILED:
					MessageBox.Show(ConstComStr.MSG_03_024, ConstComStr.CAPTION_03_024, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_PAPER_JAM:
					MessageBox.Show(ConstComStr.MSG_03_025, ConstComStr.CAPTION_03_025, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_COVER_OPEN:
					MessageBox.Show(ConstComStr.MSG_03_026, ConstComStr.CAPTION_03_026, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_MICR_NODATA:
					MessageBox.Show(ConstComStr.MSG_03_027, ConstComStr.CAPTION_03_027, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_MICR_BADDATA:
					MessageBox.Show(ConstComStr.MSG_03_028, ConstComStr.CAPTION_03_028, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_MICR_NOISE:
					MessageBox.Show(ConstComStr.MSG_03_029, ConstComStr.CAPTION_03_029, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_SCN_COMPRESS:
					MessageBox.Show(ConstComStr.MSG_03_030, ConstComStr.CAPTION_03_030, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_PAPER_EXIST:
					MessageBox.Show(ConstComStr.MSG_03_031, ConstComStr.CAPTION_03_031, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_PAPER_INSERT:
					MessageBox.Show(ConstComStr.MSG_03_032, ConstComStr.CAPTION_03_032, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_EXEC_SCAN_CHECK_CONTINUOUS:
					MessageBox.Show(ConstComStr.MSG_03_033, ConstComStr.CAPTION_03_033, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_EXEC_SCAN_CHECK_ONEBYONE:
					MessageBox.Show(ConstComStr.MSG_03_034, ConstComStr.CAPTION_03_034, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_EXEC_SCAN_IDCARD:
					MessageBox.Show(ConstComStr.MSG_03_035, ConstComStr.CAPTION_03_035, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_EXEC_PRINT_ROLLPAPER:
					MessageBox.Show(ConstComStr.MSG_03_036, ConstComStr.CAPTION_03_036, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_EXEC_PRINT_VALIDATION:
					MessageBox.Show(ConstComStr.MSG_03_037, ConstComStr.CAPTION_03_037, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_THREAD:
					MessageBox.Show(ConstComStr.MSG_03_038, ConstComStr.CAPTION_03_038, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_IMAGE_FILEOPEN:
					MessageBox.Show(ConstComStr.MSG_03_039, ConstComStr.CAPTION_03_039, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_IMAGE_UNKNOWNFORMAT:
					MessageBox.Show(ConstComStr.MSG_03_040, ConstComStr.CAPTION_03_040, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_SIZE:
					MessageBox.Show(ConstComStr.MSG_03_041, ConstComStr.CAPTION_03_041, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_NOT_FOUND:
					MessageBox.Show(ConstComStr.MSG_03_042, ConstComStr.CAPTION_03_042, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_NOT_EXEC:
					MessageBox.Show(ConstComStr.MSG_03_043, ConstComStr.CAPTION_03_043, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_BARCODE_NODATA:
					MessageBox.Show(ConstComStr.MSG_03_044, ConstComStr.CAPTION_03_044, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_MICR_PARSE:
					MessageBox.Show(ConstComStr.MSG_03_045, ConstComStr.CAPTION_03_045, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_SCN_IQA:
					MessageBox.Show(ConstComStr.MSG_03_046, ConstComStr.CAPTION_03_046, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_PRINT_DATA_LENGTH_EXCEED:
					MessageBox.Show(ConstComStr.MSG_03_047, ConstComStr.CAPTION_03_047, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_PRINT_DATA_UNRECEIVE:
					MessageBox.Show(ConstComStr.MSG_03_048, ConstComStr.CAPTION_03_048, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_INK_STATUS:
					MessageBox.Show(ConstComStr.MSG_03_050, ConstComStr.CAPTION_03_051, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				case ErrorCode.ERR_IMAGE_FILEREAD:
					MessageBox.Show(ConstComStr.MSG_03_049, ConstComStr.CAPTION_03_049, MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
				default:
					MessageBox.Show(errResult.ToString(), "ERR", MessageBoxButtons.OK, MessageBoxIcon.Warning);
					break;
			}
		}

		private string GetErrorString(ErrorCode errResultCode)
		{
			switch( errResultCode )
			{
				case ErrorCode.SUCCESS:
					return ConstComStr.CAPTION_00_000;
				case ErrorCode.ERR_TYPE:
					return ConstComStr.CAPTION_03_000;
				case ErrorCode.ERR_OPENED:
					return ConstComStr.CAPTION_03_001;
				case ErrorCode.ERR_NO_PRINTER:
					return ConstComStr.CAPTION_03_002;
				case ErrorCode.ERR_NO_TARGET:
					return ConstComStr.CAPTION_03_003;
				case ErrorCode.ERR_NO_MEMORY:
					return ConstComStr.CAPTION_03_004;
				case ErrorCode.ERR_HANDLE:
					return ConstComStr.CAPTION_03_005;
				case ErrorCode.ERR_TIMEOUT:
					return ConstComStr.CAPTION_03_006;
				case ErrorCode.ERR_ACCESS:
					return ConstComStr.CAPTION_03_007;
				case ErrorCode.ERR_PARAM:
					return ConstComStr.CAPTION_03_008;
				case ErrorCode.ERR_NOT_SUPPORT:
					return ConstComStr.CAPTION_03_009;
				case ErrorCode.ERR_OFFLINE:
					return ConstComStr.CAPTION_03_010;
				case ErrorCode.ERR_NOT_EPSON:
					return ConstComStr.CAPTION_03_011;
				case ErrorCode.ERR_WITHOUT_CB:
					return ConstComStr.CAPTION_03_012;
				case ErrorCode.ERR_BUFFER_OVER_FLOW:
					return ConstComStr.CAPTION_03_013;
				case ErrorCode.ERR_REGISTRY:
					return ConstComStr.CAPTION_03_014;
				case ErrorCode.ERR_PAPERINSERT_TIMEOUT:
					return ConstComStr.CAPTION_03_015;
				case ErrorCode.ERR_EXEC_FUNCTION:
					return ConstComStr.CAPTION_03_016;
				case ErrorCode.ERR_EXEC_MICR:
					return ConstComStr.CAPTION_03_017;
				case ErrorCode.ERR_EXEC_SCAN:
					return ConstComStr.CAPTION_03_018;
				case ErrorCode.ERR_RESET:
					return ConstComStr.CAPTION_03_019;
				case ErrorCode.ERR_ABORT:
					return ConstComStr.CAPTION_03_020;
				case ErrorCode.ERR_MICR:
					return ConstComStr.CAPTION_03_021;
				case ErrorCode.ERR_SCAN:
					return ConstComStr.CAPTION_03_022;
				case ErrorCode.ERR_LINE_OVERFLOW:
					return ConstComStr.CAPTION_03_023;
				case ErrorCode.ERR_PAPER_PILED:
					return ConstComStr.CAPTION_03_024;
				case ErrorCode.ERR_PAPER_JAM:
					return ConstComStr.CAPTION_03_025;
				case ErrorCode.ERR_COVER_OPEN:
					return ConstComStr.CAPTION_03_026;
				case ErrorCode.ERR_MICR_NODATA:
					return ConstComStr.CAPTION_03_027;
				case ErrorCode.ERR_MICR_BADDATA:
					return ConstComStr.CAPTION_03_028;
				case ErrorCode.ERR_MICR_NOISE:
					return ConstComStr.CAPTION_03_029;
				case ErrorCode.ERR_SCN_COMPRESS:
					return ConstComStr.CAPTION_03_030;
				case ErrorCode.ERR_PAPER_EXIST:
					return ConstComStr.CAPTION_03_031;
				case ErrorCode.ERR_PAPER_INSERT:
					return ConstComStr.CAPTION_03_032;
				case ErrorCode.ERR_EXEC_SCAN_CHECK_CONTINUOUS:
					return ConstComStr.CAPTION_03_033;
				case ErrorCode.ERR_EXEC_SCAN_CHECK_ONEBYONE:
					return ConstComStr.CAPTION_03_034;
				case ErrorCode.ERR_EXEC_SCAN_IDCARD:
					return ConstComStr.CAPTION_03_035;
				case ErrorCode.ERR_EXEC_PRINT_ROLLPAPER:
					return ConstComStr.CAPTION_03_036;
				case ErrorCode.ERR_EXEC_PRINT_VALIDATION:
					return ConstComStr.CAPTION_03_037;
				case ErrorCode.ERR_THREAD:
					return ConstComStr.CAPTION_03_038;
				case ErrorCode.ERR_IMAGE_FILEOPEN:
					return ConstComStr.CAPTION_03_039;
				case ErrorCode.ERR_IMAGE_UNKNOWNFORMAT:
					return ConstComStr.CAPTION_03_040;
				case ErrorCode.ERR_SIZE:
					return ConstComStr.CAPTION_03_041;
				case ErrorCode.ERR_NOT_FOUND:
					return ConstComStr.CAPTION_03_042;
				case ErrorCode.ERR_NOT_EXEC:
					return ConstComStr.CAPTION_03_043;
				case ErrorCode.ERR_BARCODE_NODATA:
					return ConstComStr.CAPTION_03_044;
				case ErrorCode.ERR_MICR_PARSE:
					return ConstComStr.CAPTION_03_045;
				case ErrorCode.ERR_SCN_IQA:
					return ConstComStr.CAPTION_03_046;
				case ErrorCode.ERR_PRINT_DATA_LENGTH_EXCEED:
					return ConstComStr.CAPTION_03_047;
				case ErrorCode.ERR_PRINT_DATA_UNRECEIVE:
					return ConstComStr.CAPTION_03_048;
				case ErrorCode.ERR_INK_STATUS:
					return ConstComStr.CAPTION_03_051;
				case ErrorCode.ERR_IMAGE_FILEREAD:
					return ConstComStr.CAPTION_03_049;
				default:
					return errResultCode.ToString();
			}
		}

		private void LoadResourceString()
		{
			////////////////
			//// Step 1 ////
			////////////////

			// Title
			this.Text               = ConstComStr.STR_SAMPLEDLG_TITLE;

			// Scan Button
			btnScan.Text            = ConstComStr.STR_SAMPLEDLG_BUTTON_SCAN;

			// ScanCancel Button
			btnScanCancel.Text      = ConstComStr.STR_SAMPLEDLG_BUTTON_SCANCANCEL;

			// Exit Button
			btnExit.Text            = ConstComStr.STR_SAMPLEDLG_BUTTON_EXIT;
		}
	}
}
